// ============================================================================
//
// Copyright (c) 2001  Lutz Kettner, see the Copyright file for details
//
// ----------------------------------------------------------------------------
//
// file          : PseudoT_items_2.h
// revision      : $Revision: 1.2 $
// revision_date : $Date: 2001/09/03 06:13:35 $
// author(s)     : Lutz Kettner
//
// Default items class for PseudoT (Vertex, Halfedge, Face).
// ============================================================================

#ifndef PTW_PSEUDOT_ITEMS_2_H
#define PTW_PSEUDOT_ITEMS_2_H 1

#include <CGAL/HalfedgeDS_vertex_base.h>
#include <CGAL/HalfedgeDS_halfedge_base.h>
#include <CGAL/HalfedgeDS_face_base.h>

namespace PTW {

template <class Refs, class Traits_>
class PseudoT_vertex_2
    : public CGAL::HalfedgeDS_vertex_base<Refs, CGAL::Tag_true,
                                          typename Traits_::Point_2> {
public:
    typedef Traits_                     Traits;
    typedef typename Traits::Point_2    Point_2;
    typedef CGAL::HalfedgeDS_vertex_base<Refs, CGAL::Tag_true, Point_2> Vbase;

    PseudoT_vertex_2() {}
    PseudoT_vertex_2( const Point_2& pt) : Vbase( pt) {}
};


template <class Refs, class Traits_>
class PseudoT_halfedge_2
    : public CGAL::HalfedgeDS_halfedge_base<Refs> {
public:
    typedef Traits_                          Traits;
    PseudoT_halfedge_2() {}
};

template <class Refs, class Traits_>
class PseudoT_face_2
    : public CGAL::HalfedgeDS_face_base<Refs, CGAL::Tag_true> {
public:
    typedef Traits_                     Traits;
    PseudoT_face_2() {}
};


class PseudoT_items_2 {
public:
    template < class Refs, class Traits>
    struct Vertex_wrapper {
        typedef PseudoT_vertex_2< Refs, Traits>    Vertex;
    };
    template < class Refs, class Traits>
    struct Halfedge_wrapper {
        typedef PseudoT_halfedge_2< Refs, Traits>  Halfedge;
    };
    template < class Refs, class Traits>
    struct Face_wrapper {
        typedef PseudoT_face_2< Refs, Traits>      Face;
    };
};

} // namespace PTW
#endif // PTW_PSEUDOT_ITEMS_2_H //
// EOF //
