// eval_pseudo.C
// ============================================================================
// revision      : $Revision: 1.1 $
// revision_date : $Date: 2001/09/03 06:12:57 $
// author(s)     : Lutz Kettner  <kettner@inf.ethz.ch>
//
// Reads trinum and evaluates data in it.
// ============================================================================

#include <CGAL/basic.h>
#include <cstring>

using namespace std;

const char* pseudo_names[11] = {
    "<none>",
    "<none>",
    "<none>",
    "data/pseudo03.b08",
    "data/pseudo04.b08",
    "data/pseudo05.b08",
    "data/pseudo06.b08",
    "data/pseudo07.b16",
    "data/pseudo08.b16",
    "data/pseudo09.b16",
    "data/pseudo10.b32"
};
int pseudo_bytes[11] = { 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 4};


// ----------------------------------------------------------------
int read( istream& in, int k) {
    int i;
    switch (k) {
    case 1:
        {
	    CGAL::UInteger8 j;
	    in.read( & j, 1);
	    i = j;
	    break;
	}
    case 2:
        {
	    CGAL::UInteger16 j;
	    in.read( & j, 2);
	    // swap byte order on big endian machines
	    #ifdef CGAL_BIG_ENDIAN 
	    char* p = reinterpret_cast<char*>( &j);
	    std::swap( p[0], p[1]);
            #endif
	    i = j;
	    break;
	}
    default:
        {
	    CGAL::UInteger32 j;
	    in.read( & j, 4);
	    // swap byte order on big endian machines
	    #ifdef CGAL_BIG_ENDIAN 
	    char* p = reinterpret_cast<char*>( &j);
	    std::swap( p[0], p[3]);
	    std::swap( p[1], p[2]);
            #endif
	    i = j;
	    break;
	}
    }
    return i;
}

// eval: read binary pseudo data and eval.
// ----------------------------------------------------------------
void eval( int n) {
    CGAL_precondition( n > 2 && n < 11);
    ifstream pseudo_in( pseudo_names[n]);
    int  min_value    = INT_MAX;
    int  max_value    = 0;
    long counter      = 0;
    long min_counter  = 0;
    long max_counter  = 0;
    while ( true) {
	int pseudo = read( pseudo_in, pseudo_bytes[n]);
	if ( ! pseudo_in)
	    break;
	if ( pseudo < min_value)
	    min_value = pseudo;
	if ( pseudo > max_value)
	    max_value = pseudo;
    }
    cerr << "min value   = " << min_value << endl;
    cerr << "max value   = " << max_value << endl;
    if ( ! pseudo_in.eof()) {
	cerr << "Error reading files." << endl;
	exit(1);
    }
    pseudo_in.close();
    pseudo_in.open( pseudo_names[n]);
    while ( true) {
	int pseudo = read( pseudo_in, pseudo_bytes[n]);
	if ( ! pseudo_in)
	    break;
	if ( pseudo == min_value) {
	    ++ min_counter;
	    if ( min_counter < 10)
		cerr << "min at pos  = " << counter << endl;
	}
	if ( pseudo == max_value) {
	    ++ max_counter;
	    if ( max_counter < 10)
		cerr << "max at pos  = " << counter << endl;
	}
	++counter;
    }
    cerr << "#min values = " << min_counter << endl;
    cerr << "#max values = " << max_counter << endl;
    if ( ! pseudo_in.eof()) {
	cerr << "Error reading files." << endl;
	exit(1);
    }
}


// main function with standard unix commandline arguments
// ----------------------------------------------------------------
int main( int argc, char **argv) {
    const int min_parameters = 1;
    const int max_parameters = 1;
    int n = 0; // number of parameters
    char *parameters[ max_parameters];
    bool help = false;
    for (int i = 1; i < argc; i++) { // check commandline options
        if ( (strcmp( "-h", argv[i]) == 0) || 
	     (strcmp( "-help", argv[i]) == 0))
            help = true;
        else if ( n < max_parameters ) {
            parameters[ n++] = argv[i];
        } else {
	    ++n;
            break;
	}
    }
    if ((n < min_parameters) || (n > max_parameters) || help) {
        if ( ! help)
            cerr << "Error: in parameter list" << endl;
        cerr << "Usage: eval_pseudo [<options>] <n>"<< endl;
	cerr << "    Evaluates ./data/trinum<n> data sets." << endl;
        cerr << "    Reports minima and maxima." << endl;
        cerr << "    All positions in the order type DB are counted beginning "
                "with 0." << endl;
        cerr << "       -h             help." << endl;
        exit( ! help);
    }
    int n_points = atoi( parameters[0]);
    if ( n_points < 3 || n > 10) {
	cerr << "Error: requires an integer in [3,10] as argument." 
	     << endl;
	exit(1);
    }
    eval( n_points);
    return 0;
}
